import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List.*;

/**
 * A World, in wich animals life.
 *
 * @author Hannes Riechert
 * @version 2008-10
 */
public class AnimalsWorld extends MyWorld
{
    private boolean firstDraw = true;
    private boolean noGraph = true;
    private boolean auto = false;
    private Diagramarea d;
    
    /**
     * The Constructor from the class AnimalsWorld
     */
    public AnimalsWorld()
    {
        super(19, 18, 30);
        setBackground("cell.jpg");
        setPaintOrder(Diagramarea.class,Animal.class);
    }
    
    /**
     * If the a-key is pressed, one ant is placed in the world.<BR>
     * If the p-key is pressed, one anteater (predator) is placed in the world.<BR>
     * If the l-key is pressed, one leaf is placed in the world.<BR>
     * If the b-key is pressed, one barrier is placed in the world.<BR>
     * If the t-key is pressed, one tree is placed in the world.<BR>
     * If the n-key is pressed, one time the populate-method is call.<BR>
     * If the mouse is klicked, and no Diagram is in the world, the diagram is placed into the world.
     */
    public void act()
    {
        if(Greenfoot.isKeyDown("a"))
            addObject(new Ant(), Greenfoot.getRandomNumber(getWidth()), Greenfoot.getRandomNumber(getHeight()));
        if(Greenfoot.isKeyDown("p"))
            addObject(new Anteater(), Greenfoot.getRandomNumber(getWidth()), Greenfoot.getRandomNumber(getHeight()));
        if(Greenfoot.isKeyDown("l"))
            addObject(new Leaf(), Greenfoot.getRandomNumber(getWidth()), Greenfoot.getRandomNumber(getHeight()));
        if(Greenfoot.isKeyDown("b"))
            addObject(new Barrier(), Greenfoot.getRandomNumber(getWidth()), Greenfoot.getRandomNumber(getHeight()));
        if(Greenfoot.isKeyDown("t"))
            addObject(new Tree(), Greenfoot.getRandomNumber(getWidth()), Greenfoot.getRandomNumber(getHeight()));
        
        if(Greenfoot.isKeyDown("n") &&! auto) {
            removeObjects(getObjects(Actor.class));
            populate();
            addObject(d, 9, 9);
            auto = true;
        }
            
        
        if(firstDraw) {
            d = new Diagramarea();
            addObject(d, 9, 9);
            firstDraw = false;
            noGraph = false;
        }
        
        if(Greenfoot.mouseClicked(null)) {
            if(getObjects(Diagramarea.class).isEmpty()) {
                addObject(d, 9, 9);
                noGraph = false;
            }
            else {
                if(! getObjectsAt(9, 9, Diagramarea.class).isEmpty())
                    removeObject(d);
                    noGraph = true;
            }
        }
        
        if(noGraph) {
            addObject(d, 9, 9);
            d.act();
            removeObject(d);
        }
    }
    
    /**
     * Populates the world with animals.
     */
    public void populate() {
        randomTrees(2);
        randomLeafs(10);
        randomAnts(30);
        randomAnteaters(5);
        randomBarriers(5);
    }
    
    /**
     * A number of leafs placed into the world.
     */
    public void randomLeafs(int number)
    {
        for (int i=0; i < number; i++) {
            Leaf l = new Leaf();
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(l, x, y);
        }
    }
    
    /**
     * A number of ants placed into the world.
     */
    public void randomAnts(int number)
    {
        for (int i=0; i < number; i++) {
            Ant a = new Ant();
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(a, x, y);
        }
    }
    
    /**
     * A number of anteaters placed into the world.
     */
    public void randomAnteaters(int number)
    {
        for (int i=0; i < number; i++) {
            Anteater a = new Anteater();
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(a, x, y);
        }
    }
    
    /**
     * A number of barriers placed into the world.
     */
    public void randomBarriers(int number)
    {
        for (int i=0; i < number; i++) {
            Barrier b = new Barrier();
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(b, x, y);
        }
    }
    
    /**
     * A number of trees placed into the world.
     */
    public void randomTrees(int number)
    {
        for (int i=0; i < number; i++) {
            Tree t = new Tree();
            int x = Greenfoot.getRandomNumber(getWidth());
            int y = Greenfoot.getRandomNumber(getHeight());
            addObject(t, x, y);
        }
    }
}